# -*- coding: utf-8 -*-
from odoo import models
from openerp.addons.itu_picking_wave.utils import dotdict


class PickingListByLocation(models.AbstractModel):
    _inherit = 'report.itu_picking_wave.picking_list_by_location'

    def get_picking_list_by_location(self, docs):
        cr = self.env.cr

        sql = """
            SELECT sp.location_id,
                sp.product_id,
                sloc.name,
                pp.default_code,
                pt.name,
                SUM(sp.product_qty) qty_todo,
                SUM(sp.qty_done) qty_done,
                uom.name as uom,
                CASE 
                    WHEN picking.state = 'cancel' then 'cancel'
                    ELSE 'not_cancel' 
                END
            FROM stock_pack_operation sp
            INNER JOIN stock_picking picking ON picking.id = sp.picking_id
            INNER JOIN stock_location sloc ON sloc.id = sp.location_id
            INNER JOIN product_uom uom ON sp.product_uom_id = uom.id
            INNER JOIN product_product pp ON pp.id = sp.product_id
            INNER JOIN product_template pt ON pp.product_tmpl_id = pt.id
            WHERE sp.picking_id IN %s
            GROUP BY sp.location_id,
                    sp.product_id,
                    sloc.name,
                    uom.name,
                    pt.name,
                    pp.default_code,
                    picking.state
            ORDER BY sp.location_id, pt.name
        """

        objects = []
        for wave in docs:
            state = self.get_state_name(wave.state)
            params = self.make_sql_list(wave.picking_ids)
            if not params:
                continue
            cr.execute(sql % (params, ))
            res = cr.fetchall()
            for x in res:
                product = x[3]
                code = x[4]
                if code:
                    product = '[%s] %s' % (product, code)
                
                o = {
                    'location_id': x[0],
                    'product_id': x[1],
                    'sloc': x[2],
                    'product_code': x[3],
                    'product_name': x[4],
                    'product': product,
                    'qty_todo': x[5],
                    'qty_done': x[6],
                    'uom': x[7],
                    'state_picking': x[8]
                    }
                oo = dotdict.dotdict(o)
                found = False
                for v in objects:
                    if (v.loc_id == x[0] and v.wave_id == wave.id):
                        if (x[8] != 'cancel'):
                            v.val.append(oo)
                        if (x[8] == 'cancel'):
                            v.cancel.append(oo)
                        found = True

                if not objects or not found:
                    obj = {
                        'wave_start': wave.date_start,
                        'wave_end': wave.date_end,
                        'wave_name': wave.name,
                        'state': state,
                        'user_id': wave.user_id.name,
                        'loc_name': x[2],
                        'loc_id': x[0],
                        'wave_id': wave.id,
                    }
                    if (x[8] != 'cancel'):
                        obj.update({'val': [oo], 'cancel': []})
                    if (x[8] == 'cancel'):
                        obj.update({'cancel': [oo], 'val': []})

                    objj = dotdict.dotdict(obj)
                    objects.append(objj)
        return objects