from odoo import api, models
from openerp.addons.itu_picking_wave.utils import dotdict


class CosolidationList(models.AbstractModel):
    _name = 'report.itu_picking_wave.consolidate_by_product'

    def make_sql_list(self, picking_ids):
        if picking_ids:
            var = '('
            for p in picking_ids:
                var += str(p.id) + ','
            var = var[:-1]
            var += ')'
            return var

    def get_state_name(self, state):
        states = [
            ('draft', 'Draft'),
            ('in_progress', 'Running'),
            ('done', 'Done'),
            ('cancel', 'Cancelled')
        ]

        for s in states:
            key, value = s
            if (key == state):
                return value

    def get_consolidate_by_product(self, docs):
        cr = self.env.cr

        sql = """
            SELECT spick.id,
                sp.location_id,
                sp.product_id,
                spick.name,
                sloc.complete_name,
                pt.name,
                pp.default_code,
                uom.name AS uom,
                SUM(sp.product_qty) qty_todo,
                SUM(sp.qty_done) qty_done
            FROM stock_pack_operation sp
            INNER JOIN stock_location sloc ON sloc.id = sp.location_id
            INNER JOIN STOCK_picking spick ON spick.id = sp.picking_id
            INNER JOIN product_uom uom ON sp.product_uom_id = uom.id
            INNER JOIN product_product pp ON pp.id = sp.product_id
            INNER JOIN product_template pt ON pp.product_tmpl_id = pt.id
            WHERE picking_id IN %s
            GROUP BY spick.id,
                    sp.location_id,
                    sp.product_id,
                    spick.name,
                    sloc.complete_name,
                    uom.name,
                    pt.name,
                    pp.default_code
            ORDER BY sp.location_id,
                    pt.name
        """

        objects = []
        for wave in docs:
            state = self.get_state_name(wave.state)
            params = self.make_sql_list(wave.picking_ids)
            if not params:
                continue
            cr.execute(sql % (params,))
            res = cr.fetchall()

            for x in res:
                product = x[5]
                code = x[6]
                if code:
                    product = '[%s] %s' % (product, code)
                o = {
                    'picking_id': x[0],
                    'location_id': x[1],
                    'product_id': x[2],
                    'picking_name': x[3],
                    'sloc': x[4],
                    'product': product,
                    'uom': x[7],
                    'qty_todo': x[8],
                    'qty_done': x[9],
                    'product_code': x[5],
                    'product_name': x[6],
                    }
                oo = dotdict.dotdict(o)
                found = False
                for v in objects:
                    if (v.product_id == x[2] and v.wave_id == wave.id):
                        v.val.append(oo)
                        found = True

                if not objects or not found:
                    obj = {
                        'wave_start': wave.date_start,
                        'wave_end': wave.date_end,
                        'wave_name': wave.name,
                        'state': state,
                        'user_id': wave.user_id.name,
                        'product': oo.product,
                        'product_id': x[2],
                        'wave_id': wave.id,
                        'val': [oo]
                    }
                    objj = dotdict.dotdict(obj)
                    objects.append(objj)

        return objects

    def _get_lot(self, parent, value):
        sql = """
            SELECT
                spo.product_id,
                spo.product_uom_id,
                lot.id,
                lot.name,
                sum(pack_lot.qty_todo) as qty_todo,
                sum(pack_lot.qty) as qty_done
            FROM stock_production_lot lot
            INNER JOIN stock_pack_operation_lot pack_lot ON lot.id = pack_lot.lot_id
            INNER JOIN stock_pack_operation spo ON spo.id = pack_lot.operation_id
            WHERE spo.picking_id = %s
            AND   spo.product_id = %s
            GROUP BY spo.product_id, spo.product_uom_id, lot.id, lot.name
            ORDER BY lot.id, lot.name
        """

        cr = self.env.cr
        cr.execute(sql % (value.picking_id, parent.product_id,))
        res = cr.fetchall()
        objects = []
        for x in res:
            o = {
                'product_id': x[0],
                'product_uom_id': x[1],
                'lot_id': x[2],
                'lot_name': x[3],
                'qty_todo': x[4],
                'qty_done': x[5],
                }
            oo = dotdict.dotdict(o)
            objects.append(oo)
        return objects

    @api.model
    def render_html(self, docids, data=None):
        report_obj = self.env['report']
        report = report_obj._get_report_from_name('itu_picking_wave.consolidate_by_product')
        docs = self.env[report.model].search([('id', 'in', docids)])
        consolidate = self.get_consolidate_by_product(docs)
        docargs = {
            'docids': docids,
            'doc_model': report.model,
            'docs': docs,
            'consolidate': consolidate,
            'lots': self._get_lot,
        }
        return report_obj.render('itu_picking_wave.consolidate_by_product', docargs)

